#pragma once

#define DOC_ATTENTION "\n\nAttention\n---------\n\n"
#define DOC_CAUTION "\n\nCaution\n-------\n\n"
#define DOC_EXAMPLES "\n\nExamples\n--------\n\n"
#define DOC_IMPORTANT "\n\nImportant\n---------\n\n"
#define DOC_NOTE "\n\nNote\n----\n\n"
#define DOC_PARAMETERS "\n\nParameters\n----------\n\n"
#define DOC_RAISES "\n\Raises\n------\n\n"
#define DOC_RETURNS "\n\nReturns\n-------\n\n"
#define DOC_SEE_ALSO "\n\nSee Also\n--------\n\n"
#define DOC_HINT "\n\nHint\n----\n\n"
#define DOC_TIP "\n\nTip\n---\n\n"
